/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * ================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://forceten.tidalwave.it
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: GeoFinder.java,v c9bf78d7704e 2010/02/14 03:07:46 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.geo;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import it.tidalwave.util.Finder;
import it.tidalwave.util.FinderSupport;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.geo.finder.spi.GeoFinderProvider;
import org.openide.util.lookup.Lookups;
import org.openide.util.Parameters;

/***********************************************************************************************************************
 *
 * @author Fabrizio Giudici
 * @version $Id: GeoFinder.java,v c9bf78d7704e 2010/02/14 03:07:46 fabrizio $
 *
 **********************************************************************************************************************/
public final class GeoFinder<T> extends FinderSupport<T>
  {
    @Nonnull
    private final Object owner;

    @CheckForNull
    private Class<?> type;

    private final List<Object> withObjects = new ArrayList<Object>();

    public GeoFinder (final @Nonnull Object owner)
      {
        super("GeoFinder for " + owner);
        this.owner = owner;
      }

    @Override @Nonnull
    public <X> GeoFinder<X> ofType (final @Nonnull Class<X> type)
      {
        this.type = type;
        return (GeoFinder<X>)this;
      }

    @Nonnull
    public GeoFinder<T> with (final @Nonnull Object object)
      {
        this.withObjects.add(object);
        return this;
      }

    @Override @Nonnull
    protected List<T> doCompute()
      {
        try
          {
            Parameters.notNull("ofType() must be called", type);
            final Finder<?> finder = GeoFinderProvider.findFinder(owner, type, Lookups.fixed(withObjects.toArray()));
            return (List<T>)finder.from(firstResult).max(maxResults).results();
          }
        catch (NotFoundException e)
          {
            return Collections.<T>emptyList();
          }     
      }
  }
